#!/usr/bin/perl -w
# -T can only be given on the command line (taint checking)
use strict; # homework by Eric Auer      see perlre for regexp
use English; # to have $MATCH $PREMATCH $POSTMATCH

# *** new 16 feb 2002: also treats other text files

# *** future versions may handle .txt.gz and .html.gz
# *** (maybe based on .gz and html magic: header and <html>)
# *** ...
# *** or could for example parse mailboxes in a way that uses
# *** subjects as titles and somehow also from/to/date/cc...

# mintermlen=3 maxtermlen=26 ...
# <-- moved this decision to the index-main part, so that
#     the context does not get trashed!

my $DENUMBER = 1; # set to kill numeric-only terms (for now: to
   # kill digits AND the like completely!!!)
my $ALPHABET = 1; # set to limit to A-Za-z_ (rest -> space)
   # this drastical measure finally cut down memory consumption
   # note that _ represents "other" and should not be stripped!
my $WARNKILL = 0; # set to get a message if possibly important
   # stuff is thrown away (currently only in the base64 filter)

my $newcr = "\n";              # set this to " " to remove "\n",
                               # set to "\n" to keep "\n" !


sub onthtml { # arg is a filename, returns content w/o HTML tags
              # *** changed to return only a reference!
  my $a =  $_[0]; # copy arg
  my $text = "";

  unless (-T $a) { # we could try .gz as second attempt!
    print STDERR "This is not even a text file <$a>\n";
    return \$text;
  }

  unless (open(FIL,"<$a")) {
    print STDERR "Cannot read file <$a>\n";
    return \$text;
  }

  my $title = "untitled";

  $/ = undef; # slurp mode - ignore line breaks
  $text = <FIL>;
  $text = "EMPTY FILE" if (! defined $text);
  $text =~ tr{\n}{ } if ($newcr ne "\n");
  close(FIL);

  if ($text =~ s/[a-zA-Z0-9\/+=]{60,}/BASE64 /g) {
       # strings like those are probably base64 - useless!
     print STDERR "killed overlong (base64?) strings in <$a>\n"
       if ($WARNKILL);
  }  # *** new function, again to save RAM...
     # *** uuencode uses further chars:
     # *** A-Z 0-9 .,^_ <>() ;:`" &%!? +-*/ @$\# []=' (26+10+28)
     # *** but we do not handle this because it could kill
     # *** stretches of html markup as well!

  if ($DENUMBER) {
    $text =~ tr/0123456789:,./ /s; # *** cut down those numerals
    # *** using spaces, not _, because the num. are gone anyway!
    # *** be careful with the &#123; removal and this!
  }

  if ($a =~ /htm[^.]*$/i) {
    if ($text =~ /[<]title[>]?(.*?)[<]\/title/si) {
      # get the title (ungreedy, in case we have a DOUBLE title!
      # (example: 9292ov.nl reiswijzer. sigh.)
      $title = $1; # not $MATCH of course
    } else { $title = "UNTITLEDHTML"; }

    $title =~ s{[&]nbsp;}{ }gsi;      # un-nonbreak spaces
    $title =~ s{[&][A-Za-z0-9#][^;]*;}
               {_}gs; # replace specials
    $title =~ tr/A-Za-z0-9 /_/c;      # replace all strange by _
    $title =~ s/[\s\n][\s\n]+/ /gs;   # compress spaces and \n
    $title =~ s/ [_ ]* / /gs;         # remove unprintables
  }

  print STDERR "File $a: $title\n";

  if ($a =~ /htm[^.]*$/i) {
    $text .= " $title"; # add the title to the body text
    # now make "title\007word" from every word in the title:
    $title = join(" title\007", split(/ /," $title") );

    $text =~ s/^.*[\/]head[>]//gsi; # remove head part + style

    $text =~ s/[<]!--.*?--[>]//gs;  # ungreedily remove remarks
                                    # (which may contain tags!)
    $text =~ s/[<][^>]*[>]//gs;     # ungreedily remove all tags

    # for testing:
    $text =~ s{[&]nbsp;}{ }gsi;     # un-nonbreak spaces
    $text =~ s{[&][a-zA-Z0-9#][^;]*;}
              {_}gs;                # replace special chars
  }

  $text =~ tr{<>}{_}; # the popular German umlauts etc.

  if ($newcr eq "\n") {
    $text =~ s/[^\S\n][^\S\n]+/ /gs; # compress spaces but not
                             # \n ...   (not (\n or nonspace))
    $text =~ s/\n[\s\n]+/\n/gs; # make multi-cr to single-crs
    $text =~ s/[\s\n]+\n/\n/gs; # make multi-cr to single-crs
  } else {
    $text =~ s/[\s\n][\s\n]+/ /gs;   # compress spaces and \n
  }
  $text =~ s/ [_ ]* / /gs;      # remove unprintable words

  if ($ALPHABET) { 
    if ($newcr eq "\n") { 
      $text =~ tr{A-Za-z\n_}{ }cs;
    } else {
      $text =~ tr{A-Za-z_}{ }cs;
    }
  } # change non-alphabet char sequences to single spaces
    # _ must be in the alphabet or we get weird effects!

  $text .= " $title";
  return \$text;
}

return 1; # make require succeed

