#!/usr/bin/perl -w
# -T can only be given on the command line (taint checking)
use strict; # homework by Eric Auer...
# push(@INC,"."); # add local path

# Indexing over a set of HTML/text files

my $THERE = "/usr/share/texmf/doc/";
                  # where to start in directory tree...
# /usr/share/texmf/doc/latex/latex2e-html/
# /home/eric/EA/WS0102/

my $logfile = "/dev/null";
my $DOWNCASE = 1; # set to one to force the index to lower case
my $MORPHO = 1;   # *** week6: set to 1 for simple morphology...
my $GZIP = 1;     # *** week6: set to 1 to gzip the index if
                  # *** gzip is installed (compresses well: lots
                  # *** of filenames repeated, lots of numbers)

my $MAXREFS = 5000; # *** only use that many chars to describe
my %blocked;        # *** the occurance of ONE term in ONE file
                    # *** (blocked keeps track of this)

my $MINTERMLEN = 4;  # shortest terms for which we store data
my $MAXTERMLEN = 30; # longest  terms for which we store data
my $CLIPWARN = 1;    # define to get warnings about too common
   # or too long terms - the too common terms are stored in
   # their N first positions, the too long terms are ignored


require "index-browse.pl";
require "index-demarkup2.pl";
require "index-deflex.pl";   # *** for week 6, simple morphology

my %hsh;     # the big hash of hashes for the index

my %filenames; # hash to save space with the filenames!

if ($THERE eq ".") {
  (chomp($THERE = `pwd`)) || ($THERE = "./");
  print STDERR "Search begins in $THERE\n";
  # I really miss a pwd function in Perl...
}

my @htmllist = split("\011",&collecthtmlfiles($THERE));

for my $i (0..$#htmllist) {
  $filenames{$htmllist[$i]} = $i; # give out file id numbers
}

  # *** week 6: we now use tab as file name separator,
  # *** because (Win)DOS uses the : for drivespecs. sigh.

foreach my $onefile (@htmllist) {

  my $content = ${&onthtml($onefile)};

  $content =~ s/[^\w\007]/ /gs;   # simplify "other" but "\007"
                               # non-word -> " "
  $content =~ s/[\s\n][\s\n]+/ /gs;   # compress spaces and \n

  my $pos = 0;

  my $fileID = $filenames{$onefile}; # save some space!

  foreach my $oneword (split(" ",$content)) {
    # if ($oneword =~ /^_*$/) { next; } # skip "nowords"
    # better count them and do not log them, everything
    # else is very confusing!

    $pos++; # we store the word numbers where the word occurs

    if (($MINTERMLEN) && (length($oneword) < $MINTERMLEN)) {
      next; # only count, but do not index the too short word
    }

    if ($oneword =~ /^_*$/) {
      next; # only count, but do not index the unprintable word
    }

    if (($MAXTERMLEN) && (length($oneword) > $MAXTERMLEN)) {
      print STDERR "too long term in <$onefile>: <$oneword>\n"
        if ($CLIPWARN);
      next; # only count, but do not index the too short word
    }

    if ($DOWNCASE == 1) {
      $oneword = lc($oneword);     # downcase if requested...
    }

    if ($MORPHO == 1) {
      $oneword = &deflex($oneword); # *** week6: morphology
    }

    if (defined $hsh{$oneword}->{$fileID}) {
      # add one more reference to that word in that file
      # *** memory clipping is new in week 6
      if ((length($hsh{$oneword}->{$fileID}) +
           length("" . $pos . "")) > $MAXREFS) {
        # error message is shown only once:
        if (! defined $blocked{$oneword}->{$fileID}) {
          print STDERR "Too many <$oneword> in <$onefile>\n"
            if ($CLIPWARN);
          $blocked{$oneword}->{$fileID} = 1;
        } 
      } else {
        $hsh{$oneword}->{$fileID} = 
          $hsh{$oneword}->{$fileID} . "," . $pos;
      }
    } else {
      # create a new sub-hash for that word in that file
      $hsh{$oneword}->{$fileID} = $pos;
    }
  }

  $hsh{"wordcount\007"}->{$fileID} = $pos;
    # store this for relevance ranking

    # (( or: $hsh{"filename\007"}->{$onefile} = $fileID; ))
  $hsh{"filename\007"}->{$fileID} = $onefile;
    # make filenames recoverable from ID numbers...
    # ( warning: this is an index file format
    #   variation as the name is non-numeric )
}

open(LOG,">$logfile") || die "cannot write log file $logfile\n";

if ($GZIP == 1) {
  unless (open(OFILE,"|gzip -c >index.txt.gz")) {
    print STDERR "This system has no gzip or there was an write"
      .   "\nerror with index.txt.gz, reverting to plain\n";
    open(OFILE,">index.txt") || die "cannot write index.txt\n";
  }
} else {
  open(OFILE,">index.txt") || die "cannot write index.txt\n";
}

print LOG "Files indexed: " . scalar @htmllist . "\n"; # *** week 6:
print OFILE "" . scalar @htmllist . "\n"; # *** week 6:
 # *** write filecount to "header" of file

foreach my $oneword (sort keys %hsh) {
  printf LOG "%20s: ", $oneword;
  print OFILE "$oneword";
  foreach my $item (sort keys %{$hsh{$oneword}}) { # confusing,
                                                   # but works!
    print LOG "<\"$item\"," . $hsh{$oneword}->{$item} . "> ";
    print OFILE ";$item," . $hsh{$oneword}->{$item};
  }
  print LOG "\n";
  print OFILE "\n";
}

close(OFILE); # this will also close the piped progs like gzip
close(LOG);
