#!/usr/bin/perl -w
# -T can only be given on the command line (taint checking)
use strict; # homework by Eric Auer: showcontext and showtitle
            # *** week 6 additions to query-complex2

# push(@INC,"."); # add local path

require "index-demarkup2.pl"; # we use &onthtml(file)
require "index-deflex.pl";    # simple morphology / stemming
# we must also use the stemming to find the right hash keys,
# sigh...

my $VERBOSEPOS  = 0;  # set to show word numbers
my $STEMCONTEXT = 0;  # set this as MORPHO in query-complex2
my $CONTEXTSPAN = 3;  # how many words before and after each hit
my $MAXPERHIT   = 50; # show the MAXPERHIT first examples of
                      # each matching keyword (at most)


# --------------------------------------------------------------


sub showcontext { # show context in a file based on terms
  my ($fileID,$file,$words,$href) = @_;
  my %thehash = %{$href};
  my %wherelist = ();
  my $conText = "";
  my $aword;


  my $medium = ${&onthtml($file)};

  $medium =~ s/[^\w\007]/ /gs;   # simplify "other" but "\007"
                                 # non-word -> " "
  $medium =~ s/[\s\n][\s\n]+/ /gs;   # compress spaces and \n

  my @content = split(" ",$medium);
  $medium = undef; # free that memory!

  if (! defined $content[0]) {
    $conText .= "- FILE READ ERROR -";
    return $conText;
  }

  unshift(@content,"FILE-START"); # make it 1 based

  $conText .= "<i>(" . $#content . "/"
           . $thehash{"wordcount:"}->{$fileID} . " words)</i> ";

  # print STDERR "building context excerpt...\n";
  # $conText .= "<!-- keywords: $words -->";

  foreach $aword (split(/ /,$words)) { # context for all words
    $aword =~ s/^[+]//g; # irrelevant for now
    if ($aword =~ /^[-]/) { next; }
    $aword = &deflex($aword) if ($STEMCONTEXT);
      # this did the stemming/simple morphology ...

    if (! defined $thehash{$aword}->{$fileID}) {
       print STDERR "I forgot where $aword is???\n";
       next;
    }

    my @where = split(/,/,$thehash{$aword}->{$fileID});
       # now we have a list where the words occurred
    my $hitcount = 0;
    foreach my $point (@where) {

       $hitcount++;
       if ($hitcount > $MAXPERHIT) {
         next;
       }

       my $left = ($point < $CONTEXTSPAN) ?
                  0 : ($point-$CONTEXTSPAN);
       my $right = $point + $CONTEXTSPAN;
       $right = $thehash{"wordcount:"}->{$fileID}
         if $right > $thehash{"wordcount:"}->{$fileID};

       for my $area ($left .. $right) { # some context
         if ($area == $point) {
           $wherelist{$area} = 2; # precise hit
         } else {
           unless ((defined $wherelist{$area}) &&
                   ($wherelist{$area} == 2)) {
             $wherelist{$area} = 1;  # add to set of interest
           }
         }
       }

    }
  }
  my $oldpoint = -42;
  foreach my $point (sort { $a <=> $b } keys %wherelist) {
    # display the selected words
    $conText .= " ... " if (($oldpoint+1) != $point);
    $content[$point] =~ tr/\007/:/
      if (defined $content[$point]);
    $conText .= "<b>" if ($wherelist{$point} == 2); # mark hits

    if ($VERBOSEPOS) {
      $conText .= (defined $content[$point])
         ? ("[" . $content[$point] . "\@" . $point . "]" )
         : "???";
    } else {
      $conText .= (defined $content[$point])
         ? $content[$point]
         : "???";
    }

    $conText .= "</b>" if ($wherelist{$point} == 2);
    $conText .= " ";
    $oldpoint = $point;
  }
  return "$conText ...\n";
}


# --------------------------------------------------------------


sub showtitle { # show title of a file (if any)
  my ($file) = @_;
  my $theTitle = "";
  my $words = 0;

  foreach my $aword (split(/ /,${&onthtml($file)})) {
    $words++;
    if ($aword =~ /^title\007/) { # only use title words
      $aword =~ s/^title\007//g;  # remove our tagging
      $theTitle .= "$aword ";
    }
  }

  if ($words < 5) {
    $theTitle .= "FILE SHORT OR READ ERROR";
  }

  if ($theTitle =~ /^[\s]*$/) {
    $theTitle = "UNTITLED DOCUMENT";
  }

  return $theTitle;
}

return 1; # make require succeed

